@@ -2,7 +2,7 @@ package Class::DBI::Plugin::Iterator;
 use strict;
 use base qw/Class::DBI::Iterator/;
 use vars qw/$VERSION $PREFETCH/;
-$VERSION = 0.10;
+$VERSION = 0.11;
 
 $PREFETCH = 3;
 
@@ -14,38 +14,8 @@ sub PREFETCH {
     $PREFETCH;
 }
 
-
-my %driver_cache;
-
-sub _get_iterator_class {
-	my $driver  = $_[1]->db_Main()->{Driver}->{Name};
-
-	return $driver_cache{$_[1]}{$driver} if($driver_cache{$_[1]}{$driver});
-
-	my ($self,$class) = @_;
-	my $subclass = __PACKAGE__;
-
-	if( $driver eq 'mysql' ){
-		$subclass .= '::mysql';
-	}
-	else{
-		$subclass .= '::subquery';
-	}
-
-	eval qq{require $subclass};
-	if($@){ $subclass = __PACKAGE__; }
-
-	$driver_cache{$_[1]}{$driver} = $subclass;
-
-	return $subclass;
-}
-
-
 sub new {
-    my($me, $them, $sth, $args, $unknown) = @_;
-    my $class = !$unknown ? $me
-                          : $me->_get_iterator_class($them);
-
+    my($me, $them, $sth, $args) = @_;
     bless {
         _class => $them,
         _sql   => $sth->{Statement},
@@ -55,7 +25,7 @@ sub new {
         _mapper => [],
         _data   => [],
         _prefetch => $them->iterator_prefetch || $me->PREFETCH,
-    }, $class;
+    }, $me;
 }
 
 sub sql {
@@ -80,7 +50,7 @@ sub prefetch {
 sub count {
     my $self = shift;
     return $self->{_count} if defined $self->{_count};
-    return $self->all->count if $self->class->iterator_count_type eq 'use all';
+    return $self->all->count if $self->class->iterator_count_type eq 'all';
 
     my $select_from_regexp = qr/(?si)^\s*SELECT\s+.+?\s+FROM\s+/;
     my $group_check_regexp = qr/(?si)\s+GROUP\s+BY\s+(.+?)(\s+HAVING\s+.+?)?(\s+ORDER\s+BY\s+.+?)?$/;
@@ -110,7 +80,7 @@ sub count {
     };
     if ($@) {
         #warn "using \$self->all->count\n";
-        $self->class->iterator_count_type('use all');
+        $self->class->iterator_count_type('all');
         $self->{_count} = $self->all->count;
     }
 
@@ -179,16 +149,14 @@ sub import {
             if $options{prefetch} and $options{prefetch} > 0;
 
     $pkg->mk_classdata('iterator_count_type');
-    my $iterator_class = __PACKAGE__;
-    my $driver_unknown = 1;
+    my $iterator_class;
     if ($options{driver}) {
         $iterator_class .= __PACKAGE__ . "::" . $options{driver};
         eval qq{require $iterator_class};
         $iterator_class = undef if $@;
-        $driver_unknown = 0;
     }
 
-    my $statement_check_regexp = $iterator_class->statement_check_regexp;
+    my $statement_check_regexp;
 
     *{"$pkg\::sth_to_objects"} = sub {
         my ($class, $sth, $args, $plugin_disable) = @_;
@@ -198,11 +166,24 @@ sub import {
             $sth = $class->$meth();
         }
 
-        return $iterator_class->new($class, $sth, $args, $driver_unknown)
-                if not $plugin_disable and
-                   not $class->plugin_iterator_disable and
-                   defined wantarray and not wantarray and
-                   $sth->{Statement} !~ $statement_check_regexp;
+        if (not $plugin_disable
+                and not $class->plugin_iterator_disable
+                and defined wantarray and not wantarray) {
+            unless ($iterator_class) {
+                my $dname = $class->db_Main->{Driver}->{Name};
+                if ($dname eq 'mysql') {
+                    $iterator_class = __PACKAGE__ . '::mysql';
+                }
+                else {
+                    $iterator_class = __PACKAGE__ . '::subquery';
+                }
+                eval qq{require $iterator_class};
+                $iterator_class = __PACKAGE__ if $@;
+            }
+            $statement_check_regexp ||= $iterator_class->statement_check_regexp;
+            return $iterator_class->new($class, $sth, $args)
+                    if $sth->{Statement} !~ $statement_check_regexp;
+        }
 
         my (%data, @rows);
         eval {
